@@ -1,10 +1,12 @@
+use std::{borrow::Cow, collections::HashMap};
+
 use twilight_model::{
     application::{
         command::{Command, CommandOption, CommandType, Number, OptionsCommandOptionData},
         interaction::application_command::InteractionChannel,
     },
     channel::Attachment,
-    guild::Role,
+    guild::{Permissions, Role},
     id::{
         marker::{AttachmentMarker, ChannelMarker, GenericMarker, RoleMarker, UserMarker},
         Id,
@@ -31,47 +33,47 @@ use super::{internal::CreateOptionData, ResolvedUser};
 ///
 /// ## Example
 /// ```
+/// # use twilight_model::guild::Permissions;
 /// use twilight_interactions::command::{CreateCommand, ResolvedUser};
 ///
 /// #[derive(CreateCommand)]
-/// #[command(name = "hello", desc = "Say hello")]
+/// #[command(name = "hello", desc = "Say hello", default_permissions = "hello_permissions")]
 /// struct HelloCommand {
 ///     /// The message to send.
 ///     message: String,
 ///     /// The user to send the message to.
 ///     user: Option<ResolvedUser>,
 /// }
+///
+/// fn hello_permissions() -> Permissions {
+///     Permissions::SEND_MESSAGES
+/// }
 /// ```
 ///
 /// ## Macro attributes
 /// The macro provide a `#[command]` attribute to provide additional information.
 ///
-/// | Attribute                | Type           | Location               | Description                                                     |
-/// |--------------------------|----------------|------------------------|-----------------------------------------------------------------|
-/// | `name`                   | `str`          | Type                   | Name of the command (required).                                 |
-/// | `desc`                   | `str`          | Type / Field / Variant | Set the subcommand name (required).                             |
-/// | `default_permission`     | `bool`         | Type                   | Whether the command should be enabled by default.               |
-/// | `rename`                 | `str`          | Field                  | Use a different option name than the field name.                |
-/// | `autocomplete`           | `bool`         | Field                  | Enable autocomplete on this field.                              |
-/// | `channel_types`          | `str`          | Field                  | Restricts the channel choice to specific types.[^channel_types] |
-/// | `max_value`, `min_value` | `i64` or `f64` | Field                  | Set the maximum and/or minimum value permitted.                 |
+/// | Attribute                | Type                | Location               | Description                                                     |
+/// |--------------------------|---------------------|------------------------|-----------------------------------------------------------------|
+/// | `name`                   | `str`               | Type                   | Name of the command (required).                                 |
+/// | `desc`                   | `str`               | Type / Field / Variant | Description of the command (required).                          |
+/// | `default_permissions`    | `fn`[^perms]      | Type                   | Default permissions required by members to run the command.     |
+/// | `dm_permission`          | `bool`              | Type                   | Whether the command can be run in DMs.                          |
+/// | `rename`                 | `str`               | Field                  | Use a different option name than the field name.                |
+/// | `name_localizations`     | `fn`[^localization] | Type / Field / Variant | Localized name of the command (optional).                       |
+/// | `desc_localizations`     | `fn`[^localization] | Type / Field / Variant | Localized description of the command (optional).                |
+/// | `autocomplete`           | `bool`              | Field                  | Enable autocomplete on this field.                              |
+/// | `channel_types`          | `str`               | Field                  | Restricts the channel choice to specific types.[^channel_types] |
+/// | `max_value`, `min_value` | `i64` or `f64`      | Field                  | Set the maximum and/or minimum value permitted.                 |
 ///
-/// ## Example
-/// ```
-/// use twilight_interactions::command::{CreateCommand, ResolvedUser};
+/// [^perms]: Path to a function that returns [`Permissions`].
 ///
-/// #[derive(CreateCommand)]
-/// #[command(name = "hello", desc = "Say hello")]
-/// struct HelloCommand {
-///     /// The message to send.
-///     message: String,
-///     /// The user to send the message to.
-///     user: Option<ResolvedUser>,
-/// }
-/// ```
+/// [^localization]: Path to a function that returns a type that implement
+/// `IntoIterator<Item = (ToString, ToString)>`. See the module documentation to
+/// learn more.
 ///
 /// [^channel_types]: List [`ChannelType`] names in snake_case separated by spaces
-///                   like `guild_text private`.
+/// like `guild_text private`.
 ///
 /// [`CommandModel`]: super::CommandModel
 /// [`ChannelType`]: twilight_model::channel::ChannelType
@@ -89,7 +91,6 @@ pub trait CreateCommand: Sized {
 /// by the derive macro. See the [module documentation](crate::command) for
 /// a list of implemented types.
 ///
-///
 /// ## Option choices
 /// This trait can be derived on enums to represent command options with
 /// predefined choices. The `#[option]` attribute must be present on each
@@ -113,10 +114,15 @@ pub trait CreateCommand: Sized {
 /// ### Macro attributes
 /// The macro provide a `#[option]` attribute to configure the generated code.
 ///
-/// | Attribute | Type                  | Location | Description                                |
-/// |-----------|-----------------------|----------|--------------------------------------------|
-/// | `name`    | `str`                 | Variant  | Set the name of the command option choice. |
-/// | `value`   | `str`, `i64` or `f64` | Variant  | Value of the command option choice.        |
+/// | Attribute            | Type                  | Location | Description                                  |
+/// |----------------------|-----------------------|----------|----------------------------------------------|
+/// | `name`               | `str`                 | Variant  | Set the name of the command option choice.   |
+/// | `name_localizations` | `fn`[^localization]   | Variant  | Localized name of the command option choice. |
+/// | `value`              | `str`, `i64` or `f64` | Variant  | Value of the command option choice.          |
+///
+/// [^localization]: Path to a function that returns a type that implement
+///                  `IntoIterator<Item = (ToString, ToString)>`. See the
+///                  [module documentation](crate::command) to learn more.
 
 pub trait CreateOption: Sized {
     /// Create a [`CommandOption`] from this type.
@@ -131,12 +137,18 @@ pub trait CreateOption: Sized {
 pub struct ApplicationCommandData {
     /// Name of the command. It must be 32 characters or less.
     pub name: String,
-    /// Description of the option. It must be 100 characters or less.
+    /// Localization dictionary for the command name. Keys should be valid locales.
+    pub name_localizations: Option<HashMap<String, String>>,
+    /// Description of the command. It must be 100 characters or less.
     pub description: String,
+    /// Localization dictionary for the command description. Keys should be valid locales.
+    pub description_localizations: Option<HashMap<String, String>>,
     /// List of command options.
     pub options: Vec<CommandOption>,
-    /// Whether the command is enabled by default when the app is added to a guild.
-    pub default_permission: bool,
+    /// Whether the command is available in DMs.
+    pub dm_permission: Option<bool>,
+    /// Default permissions required for a member to run the command.
+    pub default_member_permissions: Option<Permissions>,
     /// Whether the command is a subcommand group.
     pub group: bool,
 }
@@ -147,8 +159,11 @@ impl From<ApplicationCommandData> for Command {
             application_id: None,
             guild_id: None,
             name: item.name,
-            default_permission: Some(item.default_permission),
+            name_localizations: item.name_localizations,
+            default_member_permissions: item.default_member_permissions,
+            dm_permission: item.dm_permission,
             description: item.description,
+            description_localizations: item.description_localizations,
             id: None,
             kind: CommandType::ChatInput,
             options: item.options,
@@ -161,7 +176,9 @@ impl From<ApplicationCommandData> for CommandOption {
     fn from(item: ApplicationCommandData) -> Self {
         let data = OptionsCommandOptionData {
             description: item.description,
+            description_localizations: item.description_localizations,
             name: item.name,
+            name_localizations: item.name_localizations,
             options: item.options,
         };
 
@@ -179,6 +196,12 @@ impl CreateOption for String {
     }
 }
 
+impl<'a> CreateOption for Cow<'a, str> {
+    fn create_option(data: CreateOptionData) -> CommandOption {
+        String::create_option(data)
+    }
+}
+
 impl CreateOption for i64 {
     fn create_option(data: CreateOptionData) -> CommandOption {
         CommandOption::Integer(data.into_number(Vec::new()))
